import sys

from src.roadmapper.roadmap import Roadmap
from src.roadmapper.timelinemode import TimelineMode
from src.tests.roadmap_generators.roadmap_abc import RoadmapABC


class ColourThemeExtensive(RoadmapABC):

    def generate_and_save_as(
            self,
            file_name: str = "example.png",
    ) -> None:
        roadmap: Roadmap = self.generate()
        roadmap.draw()
        roadmap.save(file_name)

    def generate(self) -> Roadmap:
        width = 1400
        height = 1000
        mode = TimelineMode.MONTHLY
        start_date = "2023-01-01"
        number_of_items = 12
        show_generic_dates = False
        colour_theme = "DEFAULT"

        fill_colour_1 = "#FFC000"
        fill_colour_2 = "#ED7D31"
        fill_colour_2_milestone = "#843C0C"
        fill_colour_3 = "#70AD47"
        fill_colour_3_milestone = "#385723"
        fill_colour_4 = "#4472C4"
        fill_colour_4_milestone = "#162641"
        font_colour = "black"

        roadmap = Roadmap(width, height, auto_height=True, colour_theme=colour_theme, show_marker=False)

        roadmap.set_title("STRATEGY ROADMAP 2023")
        roadmap.set_subtitle("Matariki Technologies Inc.")

        roadmap.set_timeline(
            mode=mode,
            start=start_date,
            number_of_items=number_of_items,
            show_generic_dates=show_generic_dates,
            year_fill_colour="#404040",
            year_font_colour="white",
            item_fill_colour="#404040",
            item_font_colour="white",
        )

        self.add_planning_group(roadmap, fill_colour_1, font_colour)

        self.add_strategy_group(roadmap, fill_colour_2, fill_colour_2_milestone, font_colour)

        self.add_service_development_group(roadmap, fill_colour_3, fill_colour_3_milestone, font_colour)

        self.add_business_intelligence_group(roadmap, fill_colour_4, fill_colour_4_milestone, font_colour)

        roadmap.set_footer("Generated by Roadmapper")

        return roadmap

    def add_planning_group(self, roadmap, fill_colour, font_colour):
        group = roadmap.add_group("Planning", fill_colour=fill_colour, font_colour=font_colour)
        group.add_task(
            "Vision", "2023-01-01", "2023-03-31", fill_colour=fill_colour, font_colour=font_colour
        )
        task = group.add_task(
            "Goals", "2023-02-15", "2023-03-31", fill_colour=fill_colour, font_colour=font_colour
        )
        task.add_parallel_task(
            "Strategic Intent", "2023-04-01", "2023-05-31", fill_colour=fill_colour, font_colour=font_colour
        )
        task.add_parallel_task(
            "Sales Budget", "2023-06-01", "2023-07-15", fill_colour=fill_colour, font_colour=font_colour
        )
        task.add_parallel_task(
            "Release Plans", "2023-07-16", "2023-09-30", fill_colour=fill_colour, font_colour=font_colour
        )

    def add_strategy_group(self, roadmap, fill_colour, fill_colour_milestone, font_colour):
        group = roadmap.add_group("Strategy", fill_colour=fill_colour, font_colour=font_colour)
        task = group.add_task(
            "Market Analysis", "2023-02-01", "2023-03-30", fill_colour=fill_colour, font_colour=font_colour
        )
        task.add_milestone(
            "Competitor Review", "2023-03-30", fill_colour=fill_colour_milestone, font_colour=font_colour
        )
        task.add_parallel_task(
            "SWOT", "2023-04-01", "2023-04-30", fill_colour=fill_colour, font_colour=font_colour
        )
        task = group.add_task(
            "Business Model", "2023-04-01", "2023-05-31", fill_colour=fill_colour, font_colour=font_colour
        )
        task.add_milestone(
            "Price List (Draft)", "2023-06-01", fill_colour=fill_colour_milestone, font_colour=font_colour
        )
        parallel_task = task.add_parallel_task(
            "Price Research", "2023-06-01", "2023-08-05", fill_colour=fill_colour, font_colour=font_colour
        )
        parallel_task.add_milestone(
            "Price List (Final)", "2023-07-28", fill_colour=fill_colour_milestone, font_colour=font_colour
        )
        group.add_task(
            "Objectives", "2023-06-20", "2023-09-10", fill_colour=fill_colour, font_colour=font_colour
        )
        group.add_task(
            "Sales Trends Analysis", "2023-08-15", "2023-10-10", fill_colour=fill_colour, font_colour=font_colour
        )

    def add_service_development_group(self, roadmap, fill_colour, fill_colour_milestone, font_colour):
        group = roadmap.add_group(
            "Service Development", fill_colour=fill_colour, font_colour=font_colour
        )
        task = group.add_task(
            "Product Roadmap", "2023-02-15", "2023-03-31", fill_colour=fill_colour, font_colour=font_colour
        )
        parallel_task = task.add_parallel_task(
            "Development", "2023-04-01", "2023-08-30", fill_colour=fill_colour, font_colour=font_colour
        )
        parallel_task.add_milestone(
            "Alpha May 20", "2023-05-20", fill_colour=fill_colour_milestone, font_colour=font_colour
        )
        parallel_task.add_milestone(
            "Private Beta Jul 02", "2023-07-02", fill_colour=fill_colour_milestone, font_colour=font_colour
        )
        parallel_task.add_milestone(
            "Public Beta Aug 15", "2023-08-15", fill_colour=fill_colour_milestone, font_colour=font_colour
        )
        task.add_parallel_task(
            "Release Candidate", "2023-09-01", "2023-10-15", fill_colour=fill_colour, font_colour=font_colour
        )
        parallel_task = task.add_parallel_task(
            "Release To Public", "2023-10-16", "2023-12-31", fill_colour=fill_colour, font_colour=font_colour
        )
        parallel_task.add_milestone(
            "Go Live Dec 20", "2023-12-20", fill_colour=fill_colour_milestone, font_colour=font_colour
        )

    def add_business_intelligence_group(self, roadmap, fill_colour, fill_colour_milestone, font_colour):
        group = roadmap.add_group(
            "Business Intelligence", fill_colour=fill_colour, font_colour=font_colour
        )
        task = group.add_task(
            "BI Development", "2023-04-15", "2023-12-31", fill_colour=fill_colour, font_colour=font_colour
        )
        task.add_milestone(
            "Service Dashboard", "2023-05-15", fill_colour=fill_colour_milestone, font_colour=font_colour
        )
        task.add_milestone(
            "Real-Time Analytics", "2023-08-01", fill_colour=fill_colour_milestone, font_colour=font_colour
        )
        task.add_milestone(
            "Sales Dashboard", "2023-12-15", fill_colour=fill_colour_milestone, font_colour=font_colour
        )


def is_arg_given():
    return len(sys.argv) > 1


if __name__ == '__main__':
    if is_arg_given():
        example_name = sys.argv[1]
        ColourThemeExtensive().generate_and_save_as(file_name=example_name)
    else:
        ColourThemeExtensive().generate_and_save_as()
